//
// BioArray Software Environment (BASE) - homepage http://base.thep.lu.se/
// Copyright (C) 2002 Bjrn Samuelsson
//
// This file is part of BASE.
//
// BASE is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// BASE is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
#ifndef BASE_LOWESS_H
#define BASE_LOWESS_H

#include <algorithm>
#include <cmath>
#include <base_median.h>

namespace BASE
{

class lowess
{
  public:
	typedef double (* weight_func)(double);

	static double cubic_weight_func(double a);
	static double quadratic_weight_func(double a);

	// Fit a line to a scatterplot using LOWESS.
	// x_beg and y_beg point to arrays of npoints x and y values, respectively.
	// y_fit_beg will be filled with fitted y values for all x values.
	// point_fitted_beg will be filled with flags indicating for each point
	// whether it is an end point of a line segment.
	// frac is the stiffness parameter (what fraction of the points will be
	// used in the sliding window). n_iter is the number of iterations.
	// delta is the max distance in x between points used as line segment ends.
	// x_weight_func and y_weight_func are the weight functions used for the
	// sliding means for x and y.
	static void calc(double *x_beg, int npoints, double *y_beg,
		double *y_fit_beg, char *point_fitted_beg,
		const double &frac, const int n_iter, const double &delta,
		weight_func x_weight_func = cubic_weight_func,
		weight_func y_weight_func = quadratic_weight_func);

  private:
	static bool line_fit(double *x_beg, int npoints,
		double *y_beg, double &k, double &m, double *w_beg = 0);

	static bool p_lowess(double *x_beg, int npoints, double *y_beg,
		const double &p_x, double &p_y, double *temp_w_beg,
		double *yw_beg = 0,
		weight_func x_weight_func = cubic_weight_func);

	static const double yw_range_factor;
};

}

#endif
